/*
 *  configuration.c - functions parsing the configuration file
 *
 *  Copyright (C) 1998-2000 Hugo Haas & Etienne Bernard
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <pwd.h>
#include <sys/types.h>

#include "resolv.h"
#include "configuration.h"
#include "log.h"

int yyparse(void);

extern logDevice *mainLog;

/* Open the configuration file */
FILE *openConfiguration(char *filename) {
  FILE *configFile;

  configFile = fopen(filename, "r");
  if (!configFile) {
    mainLog->doLog(mainLog, "Cannot open configuration file %s. Exiting...\n", filename);
    exit(1);
  }
  return configFile;
}

/* Initialize configuration with default values */
void initConfiguration(void) {
  extern char *safeUser;
  extern unsigned int dnsExpire;
  extern unsigned short logProtocols;
  extern unsigned short resolveProtocols;
  extern unsigned short icmpFormat, tcpFormat, udpFormat;
  extern unsigned short logClosing;
  extern unsigned short useIdent;

  if (safeUser) {
	free(safeUser);
	safeUser = NULL;
  }

  dnsExpire = DNS_EXPIRE;
  logProtocols = 0;
  resolveProtocols = 0;
  icmpFormat = tcpFormat = udpFormat = LOGFORMAT_NORMAL;
  logClosing = 0;
  useIdent = 0;
}

/* Parse the configuration file */
void parseConfigFile(char *filename) {
  extern FILE *yyin;
  extern int lineNumber;
#ifdef PARSING_DEBUG
  extern int yydebug;
#endif

#ifdef PARSING_DEBUG
  yydebug = 1;
#endif

  initConfiguration();

  yyin = openConfiguration(filename);
  lineNumber = 1;
  yyparse();
}
