/*
 *  filter.h - Filtering mechanism
 *
 *  Copyright (C) 1998-2000 Hugo Haas & Etienne Bernard
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef _IPPL_FILTER_H
#define _IPPL_FILTER_H

#include "defines.h"
#include "log.h"

#define NO_ICMP_TYPE 0xff

/* Filter entry */

struct portRange_struct {
  unsigned short int rangeMin;
  unsigned short int rangeMax;
};

struct portRanges_struct {
  struct portRange_struct dst;
  struct portRange_struct src;
};

struct hostDesc_struct {
  uint32_t address;
  uint32_t mask;
  char *hostMask;
  char *hostName;
};

union logInfo_union {
  uint8_t icmpType;
  struct portRanges_struct portRanges;
};


struct filterEntry {
  short log; /* 1 for "log", 0 for "ignore" */
  short ident; /* 1 if we should use ident */
  short resolve; /* 1 if we should resolve IP addresses */
  short logFormat; /* format used for log */
  short logClosing; /* 1 to log closing TCP connections */
  /* Protocol type */
  short protocol;
  /* Information about the packet */
  union logInfo_union logInfo;
  /* Mask for remote host */
  struct hostDesc_struct fromDesc;
  /* Mask for local interface */
  struct hostDesc_struct toDesc;
  /* Next entry */
  struct filterEntry *next;
};

struct logInfo {
  short log;
  short ident;
  short resolve;
  short logClosing;
  short logFormat;
  unsigned short protocol;
  unsigned short ihl;
  uint32_t from;
  uint32_t to;
  uint16_t srcPort;
  uint16_t dstPort;
  logDevice *logDevice;
  unsigned short closing; /* Used for TCP. Is this a closing connection ? */
};

void freeFilter(void);
void refreshFilter(void);

struct logInfo testLog(const uint32_t, const uint32_t, const uint16_t, const uint16_t, const short);

#endif
