/*
 *  ident.c - TCP/IP ident protocol query
 *
 *  Copyright (C) 1999-2000 Etienne Bernard
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <signal.h>
#include <sys/time.h>
#include <sys/types.h>
#include <unistd.h>
#include <fcntl.h>
#include <netdb.h>
#include <string.h>
#include <stdio.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <errno.h>
#include <setjmp.h>

#include "defines.h"
#include "ident.h"

static sigjmp_buf identAlarm;

void timeoutAlarm(int sig) {
  siglongjmp(identAlarm, 1);
}

char *getIdent(unsigned short localPort, unsigned short remotePort,
               uint32_t localAddr, uint32_t remoteAddr) {
  unsigned int remote, local;
  int s, i, j;
  char *c;
  struct sockaddr_in sin;
  char buffer[BUFFER_SIZE], user[IDENT_LENGTH];


  s = socket(AF_INET, SOCK_STREAM, 0);
  if (s < 0) goto unknown;

  signal(SIGALRM, timeoutAlarm);
  if (sigsetjmp(identAlarm, 1) != 0) goto unknown;
  alarm(IDENT_TIMEOUT);

  sin.sin_family = AF_INET;
  sin.sin_addr.s_addr = localAddr;
  sin.sin_port = INADDR_ANY;

  if (bind(s, (struct sockaddr *)&sin, sizeof(sin)) < 0) goto unknown;

  sin.sin_family = AF_INET;
  sin.sin_addr.s_addr = remoteAddr;
  sin.sin_port = IDENT_PORT;

  if (connect(s, (struct sockaddr *)&sin, sizeof(sin)) < 0) goto unknown;

  sprintf(buffer, "%u,%u\r\n", ntohs(remotePort), ntohs(localPort));

  /* Send query */
  i = 0;
  while (i < strlen(buffer)) {
    j = write(s, buffer + i, strlen(buffer+i));
    if (j < 0 && errno != EINTR)
      goto unknown;
    else if (j > 0)
      i+=j;
  }

  /* Read answer */
  i = 0;
  *buffer = '\0';
  while ((c=strchr(buffer, '\n')) == NULL && i < sizeof(buffer) - 1) {
    j = read(s, buffer + i, sizeof(buffer) - 1 - i);
    if (j==0 || (j < 0 && errno != EINTR))
      goto unknown;
    else if (j > 0)
      i+=j; 
  }

  if (sscanf(buffer, "%u , %u : USERID :%*[^:]:%64s", &remote, &local, user) != 3 ||
      ntohs(remotePort) != remote || ntohs(localPort) != local)
    goto unknown;
  
  if ((c = strchr(user, '\r'))) *c = '\0';

  alarm(0);
  close(s);
  return strdup(user);

 unknown:
  if (s > 0) close(s);
  alarm(0);
  return strdup(IDENT_UNKNOWN);
}
